from setuptools import setup, find_packages

with open("README.rst", "r") as f:
    long_description = f.read()

setup(
    name='venv-run',
    version='0.1.1',
    author='Gustavo José de Sousa',
    author_email='gustavo.jo.sousa@gmail.com',
    description='Run commands using Python virtual environment',
    long_description=long_description,
    long_description_content_type='text/x-rst',
    url='https://github.com/guludo/venv-run',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Topic :: Software Development',
        'Topic :: Utilities',
    ],
    package_dir={'': 'src'},
    py_modules=['venvrun'],
    python_requires='~=3.5',
    entry_points={
        'console_scripts': [
            'venv-run = venvrun:run',
        ]
    },
)



from argparse import ArgumentParser, REMAINDER
from collections import OrderedDict
from glob import glob
import os.path
import platform
import re
import subprocess
import sys


def guess():
    if platform.system() != 'Windows':
        pypath = ('bin', 'python')
    else:
        pypath = ('Scripts', 'python.exe')

    exes = []

    exes.extend(glob(os.path.join('*', *pypath)))
    exes.extend(glob(os.path.join('.*', *pypath)))
    exes.append(os.path.join(os.path.curdir, *pypath))
    try:
        with open('.python-version', 'r') as f:
            for prefix in (x.strip() for x in f if not re.match(r'\s*#', x)):
                proc = subprocess.run(
                    ('pyenv', 'prefix', prefix),
                    stdout=subprocess.PIPE, stderr=subprocess.DEVNULL)
                exes.append(os.path.join(proc.stdout.decode().strip(), *pypath))
                break
    except FileNotFoundError:
        pass

    venvs = OrderedDict(
        # Absolutize, normalize, and canonicalize for deduplication
        (os.path.realpath(os.path.abspath(
            os.path.dirname(os.path.dirname(exe)))), True)
        for exe in exes
        if os.access(exe, os.X_OK)
    )

    return venvs.keys()


def run():
    parser = ArgumentParser(
        usage='%(prog)s [OPTIONS] [--] [CMD]',
        description='''
            Run command from an existing python virtual environment (that is,
            with the environment's bin directory prepended to PATH). By default
            the location of the virtual environment directory is searched from
            your current working directory and used if only one match is found.
            This behavior can be overridden with the --venv option.

            CMD contains the command line to execute. You can prepend
            CMD with -- to avoid conflict with %(prog)s own options. If CMD is
            omitted then the environment's python interpreter is run without
            arguments.

            This tool tries to guess if you want to run the python interpreter
            so that you do not need to start CMD with 'python'. For that it
            first tries to run CMD. If that fails because the executable for
            CMD can not be found and the first word of CMD begins with '-' or
            ends with '.py', then 'python' is prepended to CMD and the
            execution is retried. If you do not desire such a behavior, pass
            the --no-guess option.
        ''',
    )

    parser.add_argument('--venv',
        help='''
            Use this virtual environment instead of searching for one.
        ''',
    )

    parser.add_argument('--no-guess',
        action='store_true',
        help='''
            Do not try to prepend 'python' when execution fails because the
            command is not found.
        ''',
    )

    args, cmd_args = parser.parse_known_args()

    if not cmd_args:
        cmd_args = ['python']

    if cmd_args and cmd_args[0] == '--':
        cmd_args.pop(0)
    if not args.venv:
        venvs = guess()

        if not venvs:
            print('No virtual environments found', file=sys.stderr)
            exit(1)

        if len(venvs) > 1:
            print('More than one virtual environment found:', file=sys.stderr)
            for p in venvs:
                print('  ' + p, file=sys.stderr)
            print('Please, use the --venv option.', file=sys.stderr)
            exit(1)

        args.venv = venvs[0]
    path = ''
    if platform.system() != 'Windows':
        path = os.path.join(args.venv, 'bin')
    else:
        path = os.path.join(args.venv, 'Scripts')

    if 'PATH' in os.environ:
        os.environ['PATH'] = path + os.pathsep + os.environ['PATH']
    else:
        os.environ['PATH'] = path

    try:
        os.execvp(cmd_args[0], cmd_args)
    except FileNotFoundError:
        if args.no_guess:
            raise

        if cmd_args[0].startswith('-') or cmd_args[0].endswith('.py'):
            cmd_args.insert(0, 'python')
            os.execvp(cmd_args[0], cmd_args)
        else:
            raise


if __name__ == '__main__':
    run()



import os
import sys
import tempfile
import unittest
from subprocess import run as real_subprocess_run
from unittest.mock import patch

sys.path.append(os.path.join(os.path.dirname(__file__), os.pardir, 'src'))
import venvrun


class VenvRunTest(unittest.TestCase):

    def setUp(self):
        self.oldcwd = os.getcwd()
        os.chdir(os.path.join(os.path.dirname(__file__), 'fixture'))

    def tearDown(self):
        os.chdir(self.oldcwd)

    def testGuess(self):

        def mock_run(*args, **kwargs):
            if args[0] == ('pyenv', 'prefix', 'venv-run-testsuite'):
                return real_subprocess_run(
                    ('echo', 'pyenv/path/somewhere'), **kwargs)
            return real_subprocess_run(*args, **kwargs)

        with patch.object(venvrun.platform, "system", return_value='Linux'), \
             patch.object(venvrun.subprocess, "run", mock_run):
            venvs = venvrun.guess()
            self.assertListEqual(
                sorted(venvs),
                sorted(
                    os.path.normpath(os.path.join(os.getcwd(), x))
                    for x in (
                            'venv', '.venv', os.curdir, 'pyenv/path/somewhere'
                    ))
            )

    def testGuessDedupe(self):

        def mock_run(*args, **kwargs):
            if args[0] == ('pyenv', 'prefix', 'venv-run-testsuite'):
                return real_subprocess_run(
                    ('echo', 'venv'), **kwargs)
            return real_subprocess_run(*args, **kwargs)

        with patch.object(venvrun.platform, "system", return_value='Linux'), \
             patch.object(venvrun.subprocess, "run", mock_run):
            venvs = venvrun.guess()
            self.assertListEqual(
                sorted(venvs),
                sorted(
                    os.path.normpath(os.path.join(os.getcwd(), x))
                    for x in (
                            'venv', '.venv', os.curdir
                    ))
            )

    def testGuessDedupeSymlink(self):
        with tempfile.TemporaryDirectory(prefix="venv-run") as tempdir:
            symlink = os.path.join(tempdir, "venv")
            os.symlink(os.path.join(os.getcwd(), "venv"), symlink)

            def mock_run(*args, **kwargs):
                if args[0] == ('pyenv', 'prefix', 'venv-run-testsuite'):
                    return real_subprocess_run(
                        ('echo', symlink), **kwargs)
                return real_subprocess_run(*args, **kwargs)

            with patch.object(venvrun.platform, "system", return_value='Linux'), \
                 patch.object(venvrun.subprocess, "run", mock_run):
                venvs = venvrun.guess()
                self.assertListEqual(
                    sorted(venvs),
                    sorted(
                        os.path.normpath(os.path.join(os.getcwd(), x))
                        for x in (
                                'venv', '.venv', os.curdir
                        ))
                )
